function [W,var0,mu,LP,Cpts] = BayesMCA(X,J,al)
%BAYESMCA Bayesian Minor Components Analysis
% [W,var0,mu,LP,Cpts] = BayesMCA(X,J,al)
% Input:
%   X:  D x N matrix, training data. Each column is an data case. Each row is a feature.
%   J:  number of components.
%   al: hyper-parameter.
% Output:
%   W:    Selected minor components.
%   var0: noise variance.
%   mu:   mean of X
%   LP:   log-probability of data in the trained BayesMCA model.
%   Cpts: indices of the selected minor components.
%
% Created by Yutian Chen, 2009.

[D,N] = size(X);

n = N + al;

mu = mean(X,2);

H = (X-repmat(mu,[1,N])) / sqrt(N);

[U,G] = svd(H);

G = diag(G); G = [G; zeros(D-length(G),1)];

bt = 1;

L = (N*G.^2 + al*bt) / n; % Estimated eigenvalues

Ic = D-J+(1:J)';

sg2 = L(Ic); % sigma^2 in minor components.

var0 = sum(L(1:D-J))/(D-J); % sigma^2 in other components.

LP = -0.5*N * ( D*log(2*pi*exp(1)) + sum(log(sg2)) + (D-J)*log(var0) );

W =  diag(1 ./ sqrt(L(Ic))) * U(:,Ic)';

Cpts = Ic;
