function [W,var0,mu,LP,Cpts] = BayesXCA(X,J,al)
%BAYESXCA Bayesian eXtreme Components Analysis
% [W,var0,mu,LP,Cpts] = BayesXCA(X,J,al)
% Input:
%   X:  D x N matrix, training data. Each column is an data case. Each row is a feature.
%   J:  number of components.
%   al: hyper-parameter.
% Output:
%   W:    Selected extreme components.
%   var0: noise variance.
%   mu:   mean of X
%   LP:   log-probability of data in the trained BayesXCA model.
%   Cpts: indices of the selected extreme components.
%
% Created by Yutian Chen, 2009.

[D,N] = size(X);

n = N + al;

LogProb = zeros(J+1,1); % Log-probabilities.

Ic = zeros(J,J+1); % Indices of selected components.

var0_gap = zeros(J+1,1); % Variance of noise.

mu = mean(X,2);

H = (X-repmat(mu,[1,N])) / sqrt(N);

[U,G] = svd(H);

G = diag(G); G = [G; zeros(D-length(G),1)]; % Sample eigenvalues.

bt = 1;

L = (N*G.^2 + al*bt) / n; % Estimated eigenvalues

% Consider J + 1 choices of the extreme components.
for j=1:J+1

    Ic(:,j) = [1:j-1,D-J+j:D]';

    sg2 = L(Ic(:,j)); % sigma^2 in extreme components.

    var0_gap(j) = sum(L(j:D-J+j-1))/(D-J); % sigma^2 in other components.

    LogProb(j) = -0.5 * N * ( D*log(2*pi*exp(1)) + sum(log(sg2)) + (D-J)*log(var0_gap(j)) );
end

[LP_max,j_max] = max(LogProb); % Choose the setting with highest log-probability.

W =  diag(1 ./ sqrt(L(Ic(:,j_max)))) * U(:,Ic(:,j_max))';

var0 = var0_gap(j_max);

LP = LP_max;

Cpts = Ic(:,j_max);
