function [al_max, als, LPs ] = ChooseAl( X, J, kfold, Xtst )
% Determin the best al with cross validation
% If Xtst is provided and is not empty, al is chosen by maximum log-probability
% of Xtst.
%
% Created by Yutian Chen, 2009.

if ~exist('kfold','var')
    kfold = 3;
end
[D,N] = size(X);
cv = cvpartition(N,'k',kfold);

als = logspace(-1,3,20);
LPs = zeros(length(als), cv.NumTestSets);
LogProb = zeros(J+1,1);

for k = 1:cv.NumTestSets
    k
    Xtrn = X(:, cv.training(k));
    Xval = X(:, cv.test(k));
    N = size(Xtrn, 2);
    
    mu = mean(Xtrn,2);
    Xtrn = (Xtrn - repmat(mu, 1, N))/sqrt(N);
    
    [U, L] = svd(Xtrn);
    L = [diag(L).^2; zeros(D-min(size(L)),1)];
    
    for i = 1:length(als)
        i
        
        al = als(i);

        n = N + al;
        
        l = (N*L+al)/n;

        trS = sum(l);

        for j=1:J+1

            Ic = [1:j-1,D-J+j:D]';

            sg2 = l(Ic);

            LogProb(j) = -0.5*N * ( D*log(2*pi*exp(1)) + sum(log(sg2)) + (D-J)*log((trS - sum(sg2))/(D-J)) );%how to include terms N

        end

        [ignore, j] = max(LogProb);
%        j = J+1; % PCA

        Ic = [1:j-1,D-J+j:D]';

        W =  inv(diag(sqrt(l(Ic)))) * U(:,Ic)';

        var0 = mean(l(j:D-J+j-1));

        iSig = (eye(D)-W'*inv(W*W')*W)/var0 + W'*W;

        if exist('Xtst', 'var') && ~isempty(Xtst)
            LPs(i, k) = mean(logGaussPDF(Xtst,mu,iSig));
        else
            LPs(i, k) = mean(logGaussPDF(Xval,mu,iSig));
        end

    end
end

[ignor, idx_al] = max(mean(LPs,2));
al_max = als(idx_al);

