%DEMO Demo of running Bayesian eXtreme Components Analysis
%
% Created by Yutian Chen, 2012.

clear;clc; close all

disp('Starting');

RandStream.setDefaultStream(RandStream('mt19937ar','seed',sum(100*clock)));

% Parameters
Ntrn = 1000;
al = 20;

% Load data
load datafreyface.mat X

N = size(X, 2);

X = X(:,randperm(N));

X = X - repmat(mean(X,2),[1,N]);
X = X ./ repmat(sqrt(sum(X.^2,2)/N),[1,N]);

Xtrn = X(:,1:Ntrn);
Xtst = X(:,Ntrn+1:end);

% Plot eigenvalues.
[D,N] = size(Xtrn);

s = svd(X/sqrt(size(X,2))); s=[s;zeros(D-length(s),1)];
s = s.^2;

figure(1)
plot(log(s),'rx-'); hold on

s=svd(Xtrn/sqrt(N)); s=[s;zeros(D-length(s),1)];
s=s.^2;
plot(log(s),'bx-');
plot(log((N*s+al)/(N+al+1)),'x-g');

H = Xtrn-repmat(mean(Xtrn,2),[1,N]);
[s, lamcor, lamvar] = covshrinkKPM(H', 1);
l = eig(s);
plot(log(sort(l,'descend')),'x-k'); hold off
title('Sorted eigenvalues');
legend({'Total data', 'Training data', 'MAP estimate', 'Shrinkage'});
drawnow

% Train models
Js = floor(linspace(1,D-2,10)); % Number of components.
for k=1:length(Js)
    J = Js(k);
    fprintf('%d/%d, #components: %d, #dimensions: %d\n', k, length(Js), J, D);
        
    % Bayes XCA
    [W,var0,mu,LP,Cpts] = BayesXCA(Xtrn,J,al);
    JminorB(k) = (sum(Cpts)-(1+J)*J/2)/(D-J); % Number of minor components.
    fprintf('BXCA #minor components: %d,\t', JminorB(k));

    iSig = (eye(D)-W'/(W*W')*W)/var0 + W'*W;
    LPBXtrn(k) = mean(logGaussPDF(Xtrn,mu,iSig));
    LPBXtst(k) = mean(logGaussPDF(Xtst,mu,iSig));

    % Bayes PCA
    [W,var0,mu,LP,Cpts] = BayesPCA(Xtrn,J,al);

    iSig = (eye(D)-W'/(W*W')*W)/var0 + W'*W;
    LPBPtrn(k) = mean(logGaussPDF(Xtrn,mu,iSig));
    LPBPtst(k) = mean(logGaussPDF(Xtst,mu,iSig));

    % Shrink XCA
    [W,var0,mu,LP,Cpts] = ShrinkXCA(Xtrn,J);

    iSig = (eye(D)-W'/(W*W')*W)/var0 + W'*W;
    LPSXtrn(k) = mean(logGaussPDF(Xtrn,mu,iSig));
    LPSXtst(k) = mean(logGaussPDF(Xtst,mu,iSig));
    
    % XCA
    [W,var0,mu,LP,Cpts] = XCA(Xtrn,J);
    Jminor(k) = (sum(Cpts)-(1+J)*J/2)/(D-J);
    fprintf('XCA #minor components: %d\n', Jminor(k));
    
    iSig = (eye(D)-W'/(W*W')*W)/var0 + W'*W;
    LPXtrn(k) = mean(logGaussPDF(Xtrn,mu,iSig));
    LPXtst(k) = mean(logGaussPDF(Xtst,mu,iSig));

    % PCA
    [W,var0,mu,LP,Cpts] = PCA(Xtrn,J);

    iSig = (eye(D)-W'/(W*W')*W)/var0 + W'*W;
    LPPtrn(k) = mean(logGaussPDF(Xtrn,mu,iSig));
    LPPtst(k) = mean(logGaussPDF(Xtst,mu,iSig));

end

% Plot results.
figure(2)
plot(Js, LPXtrn, 'b-x'); hold on
plot(Js, LPBXtrn, 'g-o');
plot(Js, LPSXtrn, 'k-*');
plot(Js, LPPtrn, 'r-<');
plot(Js, LPBPtrn, 'm->');

plot(Js, LPXtst, 'b-.x'); hold on
plot(Js, LPBXtst, 'g-.o');
plot(Js, LPSXtst, 'k-.*');
plot(Js, LPPtst, 'r-.<');
plot(Js, LPBPtst, 'm-.>'); hold off
legend({'XCA','BayesXCA','ShrinkXCA','PCA','BayesPCA'}, 'Location', 'SouthEast');

xlabel('# retained directions'); ylabel('L');
title(['\alpha=',num2str(al),', #train=',num2str(N)]);

figure(3)
plot(Js, Jminor, 'b-x', Js, Js-Jminor, 'b-.*',...
        Js, JminorB, 'g-o', Js, Js-JminorB, 'g-.s');
legend({'#MC, XCA', '#PC, XCA','#MC, BayesXCA', '#PC, BayesXCA'});
xlabel('# retained directions'); ylabel('# components');
ylim([0,J+1]);
